@@ -1,6 +1,6 @@
 /*
  * Oracle Linux DTrace.
- * Copyright (c) 2019, 2025, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2019, 2026, Oracle and/or its affiliates. All rights reserved.
  * Licensed under the Universal Permissive License v 1.0 as shown at
  * http://oss.oracle.com/licenses/upl.
  */
@@ -422,12 +422,9 @@ have_helper(uint32_t func_id)
 				BPF_CALL_HELPER(func_id),
 				BPF_RETURN()
 			};
-	dtrace_difo_t	dp;
+	dtrace_difo_t	dp = { insns, NULL, NULL, ARRAY_SIZE(insns) };
 	int		fd;
 
-	dp.dtdo_buf = insns;
-	dp.dtdo_len = ARRAY_SIZE(insns);
-
 	/* If the program loads, we can use the helper. */
 	fd = dt_bpf_prog_attach(BPF_PROG_TYPE_KPROBE, 0, 0, 0, &dp,
 			      1, log, DT_BPF_LOG_SIZE_SMALL);
@@ -486,12 +483,9 @@ have_attach_type(enum bpf_prog_type ptype, enum bpf_attach_type atype,
 				BPF_MOV_IMM(BPF_REG_0, 0),
 				BPF_RETURN()
 			};
-	dtrace_difo_t	dp;
+	dtrace_difo_t	dp = { insns, NULL, NULL, ARRAY_SIZE(insns) };
 	int		pfd, tfd = -1;
 
-	dp.dtdo_buf = insns;
-	dp.dtdo_len = ARRAY_SIZE(insns);
-
 	pfd = dt_bpf_prog_attach(ptype, atype, 0, btf_id, &dp, 0, NULL, 0);
 	/* If the program load fails, we cannot iuse the attach type. */
 	if (pfd < 0)
@@ -755,7 +749,7 @@ gmap_create_buffers(dtrace_hdl_t *dtp)
 {
 	return create_gmap(dtp, "buffers", BPF_MAP_TYPE_PERF_EVENT_ARRAY,
 			   sizeof(uint32_t), sizeof(uint32_t),
-			   dtp->dt_conf.num_online_cpus);
+			   dtp->dt_conf.max_cpuid + 1);
 }
 
 /*
@@ -974,19 +968,12 @@ gmap_create_probes(dtrace_hdl_t *dtp)
 }
 
 /*
- * Create the 'usdt_names' and 'usdt_prids' BPF maps.
+ * Create the 'usdt_names' BPF map.
  *
  * 'usdt_names':  a global hash map indexed by PRID and whose value has probe
  *                name elements at fixed offsets within the value.  This map
  *                is used for get_bvar() to look up probe name elements for
  *                any prid that was created after dtrace_go().
- *
- * 'usdt_prids':  a global hash map indexed by (pid, underlying probe ID).
- *                The value is a probe ID for the overlying USDT probe and
- *                a bit mask indicating which clauses to execute for this pid.
- *
- *                For a given (pid, PRID) key, there can be at most one
- *                overlying USDT probe.
  */
 static int
 gmap_create_usdt(dtrace_hdl_t *dtp)
@@ -998,11 +985,6 @@ gmap_create_usdt(dtrace_hdl_t *dtp)
 	if (dtp->dt_usdt_namesmap_fd == -1)
 		return -1;
 
-	dtp->dt_usdt_pridsmap_fd = create_gmap(dtp, "usdt_prids", BPF_MAP_TYPE_HASH,
-	    sizeof(usdt_prids_map_key_t), sizeof(usdt_prids_map_val_t), nusdtprobes);
-	if (dtp->dt_usdt_pridsmap_fd == -1)
-		return -1;
-
 	dtp->dt_nprobes = dtp->dt_probe_id;
 
 	return 0;
@@ -1385,8 +1367,8 @@ dt_bpf_load_progs(dtrace_hdl_t *dtp, uint_t cflags)
 		if (prp->prov->impl->attach)
 			rc = prp->prov->impl->attach(dtp, prp, fd);
 
+		close(fd);
 		if (rc < 0 && !(prp->flags & DT_PROBE_FLAG_OPTIONAL)) {
-			close(fd);
 			dt_attach_error(dtp, rc,
 					prp->desc->prv, prp->desc->mod,
 					prp->desc->fun, prp->desc->prb);
