@@ -162,8 +162,6 @@ typedef struct dtrace_stmtdesc {
 #define DT_CLSFLAG_DESTRUCT		0x0020	/* destructive */
 #define DT_CLSFLAG_RETURN		0x0040	/* aggregation */
 #define DT_CLSFLAG_AGGREGATION		0x0080	/* return action */
-#define DT_CLSFLAG_USDT_INCLUDE		0x0100	/* could be used in USDT clause */
-#define DT_CLSFLAG_USDT_EXCLUDE		0x0200	/* could not be used in USDT clause */
 
 typedef int dtrace_stmt_f(dtrace_hdl_t *dtp, dtrace_prog_t *pgp,
     dtrace_stmtdesc_t *sdp, void *data);
